import os

from marshmallow import ValidationError
from typing import List

from src.models.email import NotificationType, EmailSchema
from src.util.logger import logger

REPORTING_SENDER_EMAIL = 'CB-Noname@vmware.com'

class EmailSentError(Exception):
    def __init__(self, message):
        super().__init__(message)


def fetch_monthly_email_template():
    """
    TODO: Implement API call to fetch email template.
    :return:
    """
    return '<h1> Test email Content </h1>'


def prepare_email_notification_payload(s3_url: str, org_key: str, recipients: List[str]):
    """
    TODO: Implements 'report_present'
    :param recipients:
    :param org_key:
    :param prod_env:
    :param org_id:
    :param s3_url:
    :return:
    """
    if os.getenv('ENV_CATEGORY', 'int') != 'prd':
        recipients = ['daniel.chuang@broadcom.com']
        logger.info(f'Sending to DEBUG recipients.')
    logger.info(f'Prepare email notification payload. Recipients: [{recipients}]')
    try:
        email_template = fetch_monthly_email_template()
        email = {'notificationType': NotificationType.REPORTING.value,
                 'senderEmail': REPORTING_SENDER_EMAIL,
                 'recipientEmailList': ['daniel.chuang@broadcom.com'],
                 'subject': 'Managed Detection Daily Summary',
                 'htmlContent': email_template,
                 's3URI': s3_url,
                 'reportPresent': True,
                 'orgKey': org_key
                 }
        email_schema = EmailSchema()
        res = email_schema.load(email)
        return EmailSchema().dumps(res)
    except ValidationError as err:
        logger.exception(f'Failed to generate email payload {err.messages}')
        raise err


def send_email_sqs(sqs_client, notification_queue_url: str, s3_url: str, org_key: str, recipients):
    email_payload = prepare_email_notification_payload(s3_url, org_key, recipients)
    logger.info(f'Sending SQS payload {email_payload}')
    response = sqs_client.send_message(
        QueueUrl=notification_queue_url,
        MessageBody=email_payload
    )

    if 'MessageId' in response:
        logger.info(f'[{org_key}] Sent email request to SQS')
    else:
        raise EmailSentError(f'[{org_key}] Email sent error.')
